@@ -2,7 +2,12 @@
 # This module contains most of the code of stdeb.
 #
 import re, sys, os, shutil, select
-import ConfigParser
+try:
+    # Python 2.x
+    import ConfigParser
+except ImportError:
+    # Python 3.x
+    import configparser as ConfigParser
 import subprocess
 import tempfile
 import stdeb
@@ -22,14 +27,16 @@ __all__ = ['DebianInfo','build_dsc','expand_tarball','expand_zip',
 DH_MIN_VERS = '7'       # Fundamental to stdeb >= 0.4
 DH_IDEAL_VERS = '7.4.3' # fixes Debian bug 548392
 
-PYSUPPORT_MIN_VERS = '0.8.4' # Namespace package support was added
-                             # sometime between 0.7.5ubuntu1 and
-                             # 0.8.4lenny1 (Lenny). Might be able to
-                             # back this down.
+PYTHON_ALL_MIN_VERS = '2.6.6-3'
 
-import exceptions
-class CalledProcessError(exceptions.Exception): pass
-class CantSatisfyRequirement(exceptions.Exception): pass
+try:
+    # Python 2.x
+    from exceptions import Exception
+except ImportError:
+    # Python 3.x
+    pass
+class CalledProcessError(Exception): pass
+class CantSatisfyRequirement(Exception): pass
 
 def check_call(*popenargs, **kwargs):
     retcode = subprocess.call(*popenargs, **kwargs)
@@ -37,6 +44,14 @@ def check_call(*popenargs, **kwargs):
         return
     raise CalledProcessError(retcode)
 
+if sys.version_info.major==2:
+    help_str_py2='If True, build package for python 2. (Default=True).'
+    help_str_py3='If True, build package for python 3. (Default=False).'
+else:
+    assert sys.version_info.major==3
+    help_str_py2='If True, build package for python 2. (Default=False).'
+    help_str_py3='If True, build package for python 3. (Default=True).'
+
 stdeb_cmdline_opts = [
     ('dist-dir=', 'd',
      "directory to put final built distributions in (default='deb_dist')"),
@@ -66,19 +81,25 @@ stdeb_cmdline_opts = [
     ('ignore-install-requires', 'i',
      'ignore the requirements from requires.txt in the egg-info directory'),
     ('pycentral-backwards-compatibility=',None,
-     'If True (currently the default), enable migration from old stdeb '
-     'that used pycentral'),
+     'This option has no effect, is here for backwards compatibility, and may '
+     'be removed someday.'),
     ('workaround-548392=',None,
-     'If True (currently the default), limit binary package to single Python '
-     'version, working around Debian bug 548392 of debhelper'),
-    ('force-buildsystem=',None,
-     "If True (the default), set 'DH_OPTIONS=--buildsystem=python_distutils'"),
+     'This option has no effect, is here for backwards compatibility, and may '
+     'be removed someday.'),
     ('no-backwards-compatibility',None,
-     'If True, set --pycentral-backwards-compatibility=False and '
-     '--workaround-548392=False. (Default=False).'),
+     'This option has no effect, is here for backwards compatibility, and may '
+     'be removed someday.'),
     ('guess-conflicts-provides-replaces=',None,
      'If True, attempt to guess Conflicts/Provides/Replaces in debian/control '
      'based on apt-cache output. (Default=False).'),
+    ('with-python2=',None,
+     help_str_py2),
+    ('with-python3=',None,
+     help_str_py3),
+    ('no-python2-scripts=',None,
+     'If True, do not install scripts for python 2. (Default=False).'),
+    ('no-python3-scripts=',None,
+     'If True, do not install scripts for python 3. (Default=False).'),
     ]
 
 # old entries from stdeb.cfg:
@@ -96,6 +117,8 @@ stdeb_cfg_options = [
      'debian/control Source: (Default: <source-debianized-setup-name>)'),
     ('package=',None,
      'debian/control Package: (Default: python-<debianized-setup-name>)'),
+    ('package3=',None,
+     'debian/control Package: (Default: python3-<debianized-setup-name>)'),
     ('suite=',None,
      'suite (e.g. stable, lucid) in changelog (Default: unstable)'),
     ('maintainer=',None,
@@ -115,6 +138,7 @@ stdeb_cfg_options = [
     ('stdeb-patch-file=',None,'file containing patches for stdeb to apply'),
     ('stdeb-patch-level=',None,'patch level provided to patch command'),
     ('depends=',None,'debian/control Depends:'),
+    ('depends3=',None,'debian/control Depends:'),
     ('suggests=',None,'debian/control Suggests:'),
     ('recommends=',None,'debian/control Recommends:'),
     ('xs-python-version=',None,'debian/control XS-Python-Version:'),
@@ -141,7 +165,7 @@ class NotGiven: pass
 
 def process_command(args, cwd=None):
     if not isinstance(args, (list, tuple)):
-        raise RuntimeError, "args passed must be in a list"
+        raise RuntimeError("args passed must be in a list")
     check_call(args, cwd=cwd)
 
 def recursive_hardlink(src,dst):
@@ -169,21 +193,19 @@ def recursive_hardlink(src,dst):
 
 def debianize_name(name):
     "make name acceptable as a Debian (binary) package name"
-    name = name.replace('_','')
+    name = name.replace('_','-')
     name = name.lower()
     return name
 
 def source_debianize_name(name):
     "make name acceptable as a Debian source package name"
-    name = name.replace('_','')
+    name = name.replace('_','-')
     name = name.replace('.','-')
     name = name.lower()
     return name
 
 def debianize_version(name):
     "make name acceptable as a Debian package name"
-    name = name.replace('_','-')
-
     # XXX should use setuptools' version sorting and do this properly:
     name = name.replace('.dev','~dev')
 
@@ -213,14 +235,14 @@ def get_date_822():
     if not os.path.exists(cmd):
         raise ValueError('%s command does not exist.'%cmd)
     args = [cmd,'-R']
-    result = get_cmd_stdout(args).strip()
+    result = get_cmd_stdout(args).strip().decode('ascii')
     return result
 
 def get_version_str(pkg):
     args = ['/usr/bin/dpkg-query','--show',
            '--showformat=${Version}',pkg]
     stdout = get_cmd_stdout(args)
-    return stdout.strip()
+    return stdout.strip().decode('ascii')
 
 def load_module(name,fname):
     import imp
@@ -294,7 +316,7 @@ def get_deb_depends_from_setuptools_requires(requirements, on_failure="warn"):
         cmd = subprocess.Popen(args, stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                universal_newlines=True)
-    except Exception, le:
+    except Exception as le:
         # TODO: catch rc=1 and "E: The cache directory is empty. You need to
         # run 'apt-file update' first.", and tell the user to follow those
         # instructions.
@@ -326,7 +348,7 @@ def get_deb_depends_from_setuptools_requires(requirements, on_failure="warn"):
             dd.setdefault(
                 pydist.project_name.lower(), {}).setdefault(
                 pydist, set()).add(debname)
-        except ValueError, le:
+        except ValueError as le:
             log.warn("I got an error parsing a .egg-info file named \"%s\" "
                      "from Debian package \"%s\" as a pkg_resources "
                      "Distribution: %s" % (egginfo, debname, le,))
@@ -422,7 +444,11 @@ def expand_tarball(tarball_fname,cwd=None):
 
 def expand_zip(zip_fname,cwd=None):
     "expand a zip"
-    args = ['/usr/bin/unzip',zip_fname]
+    unzip_path = '/usr/bin/unzip'
+    if not os.path.exists(unzip_path):
+        log.error('ERROR: {} does not exist'.format(unzip_path))
+        sys.exit(1)
+    args = [unzip_path, zip_fname]
     # Does it have a top dir
     res = subprocess.Popen(
         [args[0], '-l', args[1]], cwd=cwd,
@@ -471,15 +497,22 @@ def repack_tarball_with_debianized_dirname( orig_sdist_file,
     make_tarball(repacked_sdist_file,debianized_dirname,cwd=working_dir)
     shutil.rmtree(working_dir)
 
-def dpkg_source(b_or_x,arg1,arg2=None,cwd=None):
+def dpkg_source(b_or_x,arg1,cwd=None):
     "call dpkg-source -b|x arg1 [arg2]"
     assert b_or_x in ['-b','-x']
     args = ['/usr/bin/dpkg-source',b_or_x,arg1]
-    if arg2 is not None:
-        args.append(arg2)
-
     process_command(args, cwd=cwd)
 
+def dpkg_genchanges(cwd=None):
+    args = ['/usr/bin/dpkg-buildpackage',
+            '-rfakeroot',
+            '-uc', # unsigned changes
+            '-us', # unsigned source
+            '-S',  # source package
+            '-sa', # with original source archive
+            ]
+    process_command(args,cwd=cwd)
+
 def apply_patch(patchfile,cwd=None,posix=False,level=0):
     """call 'patch -p[level] [--posix] < arg1'
 
@@ -534,7 +567,7 @@ def parse_vals(cfg,section,option):
     """parse comma separated values in debian control file style from .cfg"""
     try:
         vals = cfg.get(section,option)
-    except ConfigParser.NoSectionError, err:
+    except ConfigParser.NoSectionError as err:
         if section != 'DEFAULT':
             vals = cfg.get('DEFAULT',option)
         else:
@@ -636,16 +669,16 @@ class DebianInfo:
                  long_description=NotGiven,
                  patch_file=None,
                  patch_level=None,
-                 install_requires=None,
                  setup_requires=None,
                  debian_version=None,
-                 workaround_548392=None,
-                 force_buildsystem=None,
                  have_script_entry_points = None,
-                 pycentral_backwards_compatibility=None,
                  use_setuptools = False,
                  guess_conflicts_provides_replaces = False,
                  sdist_dsc_command = None,
+                 with_python2 = None,
+                 with_python3 = None,
+                 no_python2_scripts = None,
+                 no_python3_scripts = None,
                  ):
         if cfg_files is NotGiven: raise ValueError("cfg_files must be supplied")
         if module_name is NotGiven: raise ValueError(
@@ -679,18 +712,19 @@ class DebianInfo:
             # Allow distutils commands to override config files (this lets
             # command line options beat file options).
             for longopt, shortopt, desc in stdeb_cfg_options:
-                name = longopt[:-1]
-                name = name.replace('-','_')
+                opt_name = longopt[:-1]
+                name = opt_name.replace('-','_')
                 value = getattr( sdist_dsc_command, name )
                 if value is not None:
                     if not cfg.has_section(module_name):
                         cfg.add_section(module_name)
-                    cfg.set( module_name, name, value )
+                    cfg.set( module_name, opt_name, value )
 
         self.stdeb_version = __stdeb_version__
         self.module_name = module_name
         self.source = parse_val(cfg,module_name,'Source')
         self.package = parse_val(cfg,module_name,'Package')
+        self.package3 = parse_val(cfg,module_name,'Package3')
         forced_upstream_version = parse_val(cfg,module_name,
                                             'Forced-Upstream-Version')
         if forced_upstream_version == '':
@@ -730,22 +764,37 @@ class DebianInfo:
 
         build_deps = []
         if use_setuptools:
-            build_deps.append('python-setuptools (>= 0.6b3)')
-	if setup_requires is not None and len(setup_requires):
+            if with_python2:
+                build_deps.append('python-setuptools (>= 0.6b3)')
+            if with_python3:
+                build_deps.append('python3-setuptools')
+        if setup_requires is not None and len(setup_requires):
             build_deps.extend(
                 get_deb_depends_from_setuptools_requires(setup_requires))
 
         depends = ['${misc:Depends}', '${python:Depends}']
+        depends3 = ['${misc:Depends}', '${python3:Depends}']
         need_custom_binary_target = False
 
-        self.do_pycentral_removal_preinst = pycentral_backwards_compatibility
-
         if has_ext_modules:
             self.architecture = 'any'
+            if with_python2:
+                build_deps.append('python-all-dev (>= %s)'%PYTHON_ALL_MIN_VERS)
             depends.append('${shlibs:Depends}')
-            build_deps.append('python-all-dev')
+
+            self.architecture3 = 'any'
+            if with_python3:
+                build_deps.append('python3-all-dev')
+            depends3.append('${shlibs:Depends}')
+
         else:
             self.architecture = 'all'
+            if with_python2:
+                build_deps.append('python-all (>= %s)'%PYTHON_ALL_MIN_VERS)
+
+            self.architecture3 = 'all'
+            if with_python3:
+                build_deps.append('python3-all')
 
         self.copyright_file = parse_val(cfg,module_name,'Copyright-File')
         self.mime_file = parse_val(cfg,module_name,'MIME-File')
@@ -772,14 +821,13 @@ class DebianInfo:
                 '%s usr/share/applications'%mime_desktop_file)
 
         depends.extend(parse_vals(cfg,module_name,'Depends') )
-	if install_requires is not None and len(install_requires):
-            depends.extend(get_deb_depends_from_setuptools_requires(
-                install_requires))
+        depends3.extend(parse_vals(cfg,module_name,'Depends3') )
         self.depends = ', '.join(depends)
+        self.depends3 = ', '.join(depends3)
 
         self.debian_section = parse_val(cfg,module_name,'Section')
 
-        self.description = description
+        self.description = re.sub('\s+', ' ', description).strip()
         if long_description != 'UNKNOWN':
             ld2=[]
             for line in long_description.split('\n'):
@@ -794,15 +842,10 @@ class DebianInfo:
             self.long_description = ''
 
         if have_script_entry_points:
-            if workaround_548392:
-                build_deps.append( 'debhelper (>= %s)'%DH_MIN_VERS)
-            else:
-                build_deps.append( 'debhelper (>= %s)'%DH_IDEAL_VERS )
+            build_deps.append( 'debhelper (>= %s)'%DH_IDEAL_VERS )
         else:
             build_deps.append( 'debhelper (>= %s)'%DH_MIN_VERS )
 
-        build_deps.append('python-support (>= %s)'%PYSUPPORT_MIN_VERS)
-
         build_deps.extend( parse_vals(cfg,module_name,'Build-Depends') )
         self.build_depends = ', '.join(build_deps)
 
@@ -839,60 +882,10 @@ class DebianInfo:
                 self.patch_level = 0
 
         xs_python_version = parse_vals(cfg,module_name,'XS-Python-Version')
-        if have_script_entry_points and workaround_548392:
-
-            # Trap cases that might trigger Debian bug #548392 and
-            # workaround. Disable this block once the bugfix has
-            # become widespread and change Build-Depends: to include
-            # sufficiently recent debhelper.
-
-            if len(xs_python_version)==0:
-                # No Python version specified. For now, just use default Python
-                log.warn('working around Debian #548392, changing '
-                         'XS-Python-Version: to \'current\'')
-                xs_python_version = ['current']
-            else:
-
-                # The user specified a Python version. Check if s/he
-                # specified more than one. (Specifying a single
-                # version won't trigger the bug.)
-
-                pyversions_fname = '/usr/bin/pyversions'
-                assert os.path.exists(pyversions_fname)
-                pyversions = load_module('pyversions',pyversions_fname)
-                vstring = ', '.join(xs_python_version)
-                pyversions_result = pyversions.parse_versions(vstring)
-                if ('versions' in pyversions_result and
-                    len(pyversions_result['versions'])>1):
-
-                    vers = list(pyversions_result['versions'])
-                    # More than one Python version specified.
-
-                    # This is dubious as the following comparison
-                    # happens at source build time, but what matters
-                    # is what runs when building the binary package.
-
-                    default_vers = pyversions.default_version(version_only=True)
-                    if default_vers in vers:
-                        log.warn('working around Debian #548392, changing '
-                                 'XS-Python-Version: to \'current\'')
-                        xs_python_version = ['current']
-                    else:
-                        vers.sort()
-                        log.warn('working around Debian #548392, changing '
-                                 'XS-Python-Version: to \'%s\''%vers[-1])
-                        xs_python_version = [vers[-1]]
-                elif 'all' in pyversions_result:
-                    log.warn('working around Debian #548392, changing '
-                             'XS-Python-Version: to \'current\'')
-                    xs_python_version = ['current']
 
         if len(xs_python_version)!=0:
-            self.source_stanza_extras += ('XS-Python-Version: '+
+            self.source_stanza_extras += ('X-Python-Version: '+
                                           ', '.join(xs_python_version)+'\n')
-        self.package_stanza_extras = """\
-XB-Python-Version: ${python:Versions}
-"""
 
         dpkg_shlibdeps_params = parse_val(
             cfg,module_name,'dpkg-shlibdeps-params')
@@ -947,12 +940,16 @@ XB-Python-Version: ${python:Versions}
             provides = list(set(provides))
             replaces = list(set(replaces))
 
+        self.package_stanza_extras = ''
+        self.package_stanza_extras3 = ''
+
         if len(conflicts):
             self.package_stanza_extras += ('Conflicts: '+
                                               ', '.join( conflicts )+'\n')
 
-        provides.insert(0, 'Provides: ${python:Provides}')
-        self.package_stanza_extras += ', '.join( provides  )+'\n'
+        if len(provides):
+            self.package_stanza_extras += ('Provides: '+
+                                             ', '.join( provides  )+'\n')
 
         if len(replaces):
             self.package_stanza_extras += ('Replaces: ' +
@@ -965,10 +962,78 @@ XB-Python-Version: ${python:Versions}
 
         self.dirlist = ""
 
+        if not (with_python2 or with_python3):
+            raise RuntimeError('nothing to do - neither Python 2 or 3.')
+        sequencer_with = []
+        if with_python2:
+            sequencer_with.append('python2')
+        if with_python3:
+            sequencer_with.append('python3')
+        num_binary_packages = len(sequencer_with)
+
+        no_script_lines=[]
+
+        if no_python2_scripts:
+            # install to a location where debian tools do not find them
+            self.no_python2_scripts_cli_args = '--install-scripts=/trash'
+            no_script_lines.append(
+                'rm -rf debian/%s/trash'%(self.package,))
+        else:
+            self.no_python2_scripts_cli_args = ''
+        if no_python3_scripts:
+            # install to a location where debian tools do not find them
+            self.no_python3_scripts_cli_args = '--install-scripts=/trash'
+            no_script_lines.append(
+                'rm -rf debian/%s/trash'%(self.package3,))
+        else:
+            self.no_python3_scripts_cli_args = ''
+
+        self.scripts_cleanup = '\n'.join(['        '+s for s in no_script_lines])
+
+        if sys.prefix != '/usr':
+            # virtualenv will set distutils --prefix=/path/to/virtualenv, but
+            # we want to install into /usr.
+            workaround_virtualenv_distutils = True
+            self.install_prefix = '--prefix=/usr'
+        else:
+            workaround_virtualenv_distutils = False
+            self.install_prefix = ''
+
+        rules_override_clean_target_pythons = []
+        rules_override_build_target_pythons = []
+        rules_override_install_target_pythons = []
+        if with_python2:
+            rules_override_clean_target_pythons.append(
+                RULES_OVERRIDE_CLEAN_TARGET_PY2%self.__dict__
+                )
+            rules_override_build_target_pythons.append(
+                RULES_OVERRIDE_BUILD_TARGET_PY2%self.__dict__
+                )
+            rules_override_install_target_pythons.append(
+                RULES_OVERRIDE_INSTALL_TARGET_PY2%self.__dict__
+                )
+        if with_python3:
+            rules_override_clean_target_pythons.append(
+                RULES_OVERRIDE_CLEAN_TARGET_PY3%self.__dict__
+                )
+            rules_override_build_target_pythons.append(
+                RULES_OVERRIDE_BUILD_TARGET_PY3%self.__dict__
+                )
+            rules_override_install_target_pythons.append(
+                RULES_OVERRIDE_INSTALL_TARGET_PY3%self.__dict__
+                )
+        self.rules_override_clean_target_pythons = '\n'.join(rules_override_clean_target_pythons)
+        self.rules_override_build_target_pythons = '\n'.join(rules_override_build_target_pythons)
+        self.rules_override_install_target_pythons = '\n'.join(rules_override_install_target_pythons)
+
+        self.override_dh_auto_clean = RULES_OVERRIDE_CLEAN_TARGET%self.__dict__
+        self.override_dh_auto_build = RULES_OVERRIDE_BUILD_TARGET%self.__dict__
+        self.override_dh_auto_install = RULES_OVERRIDE_INSTALL_TARGET%self.__dict__
+        sequencer_options = ['--with '+','.join(sequencer_with)]
+        sequencer_options.append('--buildsystem=python_distutils')
+        self.sequencer_options = ' '.join(sequencer_options)
+
         setup_env_vars = parse_vals(cfg,module_name,'Setup-Env-Vars')
-        if force_buildsystem:
-            setup_env_vars.append('DH_OPTIONS=--buildsystem=python_distutils')
-        self.force_buildsystem = force_buildsystem
         self.exports = ""
         if len(setup_env_vars):
             self.exports += '\n'
@@ -990,6 +1055,21 @@ XB-Python-Version: ${python:Versions}
         else:
             self.binary_target_lines = ''
 
+        if with_python2:
+            self.control_py2_stanza = CONTROL_PY2_STANZA%self.__dict__
+        else:
+            self.control_py2_stanza = ''
+
+        if with_python3:
+            self.control_py3_stanza = CONTROL_PY3_STANZA%self.__dict__
+        else:
+            self.control_py3_stanza = ''
+
+        self.with_python2 = with_python2
+        self.with_python3 = with_python3
+        self.no_python2_scripts = no_python2_scripts
+        self.no_python3_scripts = no_python3_scripts
+
     def _make_cfg_defaults(self,
                            module_name=NotGiven,
                            default_distribution=NotGiven,
@@ -1014,6 +1094,9 @@ XB-Python-Version: ${python:Versions}
                 elif value == 'python-<debianized-setup-name>':
                     assert key=='package'
                     value = 'python-' + debianize_name(module_name)
+                elif value == 'python3-<debianized-setup-name>':
+                    assert key=='package3'
+                    value = 'python3-' + debianize_name(module_name)
                 elif value == '<setup-maintainer-or-author>':
                     assert key=='maintainer'
                     value = guess_maintainer
@@ -1086,21 +1169,10 @@ def build_dsc(debinfo,
     for fname in ['Makefile','makefile']:
         if os.path.exists(os.path.join(fullpath_repackaged_dirname,fname)):
             sys.stderr.write('*'*1000 + '\n')
-            if debinfo.force_buildsystem:
-                sys.stderr.write('WARNING: a Makefile exists in this package. '
-                                 'stdeb will tell debhelper 7 to use setup.py '
-                                 'to build and install the package, and the '
-                                 'Makefile will be ignored. You can disable '
-                                 'this behavior with the '
-                                 '--force-buildsystem=False argument to the '
-                                 'stdeb command.\n')
-            else:
-                sys.stderr.write('WARNING: a Makefile exists in this package. '
-                                 'debhelper 7 will attempt to use this rather '
-                                 'than setup.py to build and install the '
-                                 'package. You can disable this behavior with '
-                                 'the --force-buildsystem=True argument to the '
-                                 'stdeb command.\n')
+            sys.stderr.write('WARNING: a Makefile exists in this package. '
+                             'stdeb will tell debhelper 7 to use setup.py '
+                             'to build and install the package, and the '
+                             'Makefile will be ignored.\n')
             sys.stderr.write('*'*1000 + '\n')
 
 
@@ -1139,7 +1211,7 @@ def build_dsc(debinfo,
     fd = open( rules_fname, mode='w')
     fd.write(rules)
     fd.close()
-    os.chmod(rules_fname,0755)
+    os.chmod(rules_fname,0o755)
 
     #    D. debian/compat
     fd = open( os.path.join(debian_dir,'compat'), mode='w')
@@ -1168,13 +1240,6 @@ def build_dsc(debinfo,
         link_func( debinfo.copyright_file,
                  os.path.join(debian_dir,'copyright'))
 
-    #    G. debian/<package>.preinst
-    if debinfo.do_pycentral_removal_preinst:
-        preinst = PREINST%debinfo.__dict__
-        fd = open( os.path.join(debian_dir,'%s.preinst'%debinfo.package), mode='w')
-        fd.write(preinst)
-        fd.close()
-
     #    H. debian/<package>.install
     if len(debinfo.install_file_lines):
         fd = open( os.path.join(debian_dir,'%s.install'%debinfo.package), mode='w')
@@ -1192,7 +1257,11 @@ def build_dsc(debinfo,
     #    J. debian/source/format
     os.mkdir(os.path.join(debian_dir,'source'))
     fd = open( os.path.join(debian_dir,'source','format'), mode='w')
-    fd.write('1.0\n')
+    fd.write('3.0 (quilt)\n')
+    fd.close()
+
+    fd = open( os.path.join(debian_dir,'source','options'), mode='w')
+    fd.write('extend-diff-ignore="\.egg-info$"')
     fd.close()
 
     if debian_dir_only:
@@ -1220,18 +1289,14 @@ def build_dsc(debinfo,
             orig_dirname = orig_tarball_top_contents[0]
             fullpath_orig_dirname = os.path.join(tmp_dir,orig_dirname)
 
-            #    C. move original repackaged tree to .orig
-            target = fullpath_repackaged_dirname+'.orig'
-            if os.path.exists(target):
-                # here from previous invocation, probably
-                shutil.rmtree(target)
-            os.rename(fullpath_orig_dirname,target)
+            #    C. remove original repackaged tree
+            shutil.rmtree(fullpath_orig_dirname)
 
         finally:
             shutil.rmtree(tmp_dir)
 
     if 1:
-        # check versions of debhelper and python-support
+        # check versions of debhelper and python-all
         debhelper_version_str = get_version_str('debhelper')
         if len(debhelper_version_str)==0:
             log.warn('This version of stdeb requires debhelper >= %s, but you '
@@ -1245,18 +1310,18 @@ def build_dsc(debinfo,
                          'compatible with older versions of debhelper.'%(
                     DH_MIN_VERS,))
 
-        pysupport_version_str = get_version_str('python-support')
-        if len(pysupport_version_str)==0:
-            log.warn('This version of stdeb requires python-support >= %s, '
-                     'but you do not have python-support installed. '
-                     'Could not check compatibility.'%PYSUPPORT_MIN_VERS)
+        python_defaults_version_str = get_version_str('python-all')
+        if len(python_defaults_version_str)==0:
+            log.warn('This version of stdeb requires python-all >= %s, '
+                     'but you do not have this package installed. '
+                     'Could not check compatibility.'%PYTHON_ALL_MIN_VERS)
         else:
             if not dpkg_compare_versions(
-                pysupport_version_str, 'ge', PYSUPPORT_MIN_VERS ):
-                log.warn('This version of stdeb requires python-support >= %s. '
-                         'Use stdeb 0.3.x to generate source packages '
-                         'compatible with older versions of python-support.'%(
-                    PYSUPPORT_MIN_VERS,))
+                python_defaults_version_str, 'ge', PYTHON_ALL_MIN_VERS):
+                log.warn('This version of stdeb requires python-all >= %s. '
+                         'Use stdeb 0.6.0 or older to generate source packages '
+                         'that use python-support.'%(
+                    PYTHON_ALL_MIN_VERS,))
 
     #    D. restore debianized tree
     os.rename(fullpath_repackaged_dirname+'.debianized',
@@ -1271,9 +1336,10 @@ def build_dsc(debinfo,
         dist_dir))
 
     dpkg_source('-b',repackaged_dirname,
-                repackaged_orig_tarball,
                 cwd=dist_dir)
 
+    dpkg_genchanges(cwd=fullpath_repackaged_dirname)
+
     if 1:
         shutil.rmtree(fullpath_repackaged_dirname)
 
@@ -1289,8 +1355,15 @@ Maintainer: %(maintainer)s
 %(uploaders)sSection: %(debian_section)s
 Priority: optional
 Build-Depends: %(build_depends)s
-Standards-Version: 3.8.4
+Standards-Version: 3.9.1
 %(source_stanza_extras)s
+
+%(control_py2_stanza)s
+
+%(control_py3_stanza)s
+"""
+
+CONTROL_PY2_STANZA = """
 Package: %(package)s
 Architecture: %(architecture)s
 Depends: %(depends)s
@@ -1298,28 +1371,57 @@ Depends: %(depends)s
 %(long_description)s
 """
 
+CONTROL_PY3_STANZA = """
+Package: %(package3)s
+Architecture: %(architecture3)s
+Depends: %(depends3)s
+%(package_stanza_extras3)sDescription: %(description)s
+%(long_description)s
+"""
+
 RULES_MAIN = """\
 #!/usr/bin/make -f
 
 # This file was automatically generated by stdeb %(stdeb_version)s at
 # %(date822)s
-
-# Unset the environment variables set by dpkg-buildpackage. (This is
-# necessary because distutils is brittle with compiler/linker flags
-# set. Specifically, packages using f2py will break without this.)
-unexport CPPFLAGS
-unexport CFLAGS
-unexport CXXFLAGS
-unexport FFLAGS
-unexport LDFLAGS
 %(exports)s
-
 %(percent_symbol)s:
-        dh $@
+        dh $@ %(sequencer_options)s
+
+%(override_dh_auto_clean)s
+
+%(override_dh_auto_build)s
+
+%(override_dh_auto_install)s
 
 %(binary_target_lines)s
 """
 
+RULES_OVERRIDE_CLEAN_TARGET_PY2 = "        python setup.py clean -a"
+RULES_OVERRIDE_CLEAN_TARGET_PY3 = "        python3 setup.py clean -a"
+RULES_OVERRIDE_CLEAN_TARGET = """
+override_dh_auto_clean:
+%(rules_override_clean_target_pythons)s
+        find . -name \*.pyc -exec rm {} \;
+"""
+
+RULES_OVERRIDE_BUILD_TARGET_PY2 = "        python setup.py build --force"
+RULES_OVERRIDE_BUILD_TARGET_PY3 = "        python3 setup.py build --force"
+RULES_OVERRIDE_BUILD_TARGET = """
+override_dh_auto_build:
+%(rules_override_build_target_pythons)s
+"""
+
+RULES_OVERRIDE_INSTALL_TARGET_PY2 = "        python setup.py install --force --root=debian/%(package)s --no-compile -O0 --install-layout=deb %(install_prefix)s %(no_python2_scripts_cli_args)s"
+
+RULES_OVERRIDE_INSTALL_TARGET_PY3 = "        python3 setup.py install --force --root=debian/%(package3)s --no-compile -O0 --install-layout=deb %(install_prefix)s %(no_python3_scripts_cli_args)s"
+
+RULES_OVERRIDE_INSTALL_TARGET = """
+override_dh_auto_install:
+%(rules_override_install_target_pythons)s
+%(scripts_cleanup)s
+"""
+
 RULES_BINARY_TARGET = """
 binary: binary-arch binary-indep
 """
@@ -1339,19 +1441,3 @@ binary-indep: build
 %(dh_installmime_indep_line)s
 %(dh_desktop_indep_line)s
 """
-
-PREINST = """#! /bin/sh
-
-set -e
-
-# This was added by stdeb to workaround Debian #479852. In a nutshell,
-# pycentral does not remove normally remove its symlinks on an
-# upgrade. Since we're using python-support, however, those symlinks
-# will be broken. This tells python-central to clean up any symlinks.
-if [ -e /var/lib/dpkg/info/%(package)s.list ] && which pycentral >/dev/null 2>&1
-then
-    pycentral pkgremove %(package)s
-fi
-
-#DEBHELPER#
-"""
